/*****************************************************************************
**	File:	EALISTD.C
**
**	Description:
**
** 	Sample DOS client application that lists the Extended Attributes of
**		a file.
**
**	Disclaimer:
**
**		Novell, Inc. makes no representations or warranties with respect to
**		any NetWare software, and specifically disclaims any express or
**		implied warranties of merchantability, title, or fitness for a
**		particular purpose.  
**
**		Distribution of any NetWare software is forbidden without the
**		express written consent of Novell, Inc.  Further, Novell reserves
**		the right to discontinue distribution of any NetWare software.
**
**		Novell is not responsible for lost profits or revenue, loss of use
**		of the software, loss of data, costs of re-creating lost data, the
**		cost of any substitute equipment or program, or claims by any party
**		other than you.  Novell strongly recommends a backup be made before
**		any software is installed.   Developer support for this software
**		may be provided at the discretion of Novell.
**
**	Programmers:
**
**		Ini	Who					Firm
**		-----------------------------------------------------------------------
**		ABJ	Adam B. Jerome		Novell Developer Support.
**
**	History:
**
**		When		Who	What
**		-----------------------------------------------------------------------
**		01-12-95	ABJ	First code.
**
*/

/****************************************************************************
**	Compiler Setup.
*/
	/*------------------------------------------------------------------------
	**	ANSI
	*/
	#include <stdio.h>
	#include <stdlib.h>
	#include <string.h>
	#include <ctype.h>

	/*------------------------------------------------------------------------
	**	Novell
	*/
	#include <nwcalls.h>

	/*------------------------------------------------------------------------
	**	TEST
	*/
	#define NAME_SPACE		NW_NS_DOS
	#define EA_BUFFER_SIZE	512

/****************************************************************************
**	Print a buffer, Debug style.
*/
void Debug(long len, char *buffer)
	{
	char szTemp[80+1];
	long nCnt;
	int  lineCnt;

	printf("       |  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F  0123456789ABCDEF\n");
	printf("-------+------------------------------------------------- ----------------\n");

	lineCnt=0;
	for(nCnt=0; nCnt<len; ++nCnt)
		{
		if(!(nCnt % 16))
			{
			if(nCnt)
				{
				++lineCnt;
				printf("  %s\n", szTemp);
				}

			printf("0x%03Xn |", lineCnt);
			memset(szTemp, '\0', sizeof(szTemp));
			}

		printf(" %02X", 0x00FF & buffer[nCnt]);
		szTemp[strlen(szTemp)] = (char)((isprint(buffer[nCnt])) ? buffer[nCnt] : '.');
		}

	while((nCnt % 16))
		{
		printf("   ");
		++nCnt;
		}
	printf("  %s\n", szTemp);

	return;
	}

/****************************************************************************
**	Program Start.
*/
void main(int argC, char *argV[])
	{
	NWCCODE	 		rVal;								/* Used to record function status return codes. */

   NWCONN_HANDLE	fileCon;					  		/* Fileserver connection where file is stored. */
	NWDIR_HANDLE	fileDirHandle;		  			/* Directory handle pointing to where file is stored. */
   char				filePath[255+1];				/*	Path and filename of file (in relation to the above) */
	NW_IDX 			fileIdx;							/* NetWare structure to identify a file. */		
	
	int				eaCnt;							/*	Used to count the EAs of a file. */
   NWEA 				eaHandle;      				/* Handle for extended attributes context. */
   BYTE 				eaBuffer[EA_BUFFER_SIZE];	/* Buffer for extended attribute. */
   NWSIZE			eaSize;          				/* Total size of EA. */
   NWSIZE			eaScoop;         				/* Amount of EA read or written. */
	char   			eaName[255+1];					/* Name of EA record. */
	NWEA_SCAN 		eaFfStruct;						/* Used to pass info from EA findFirst, findNext, etc. */

	/*------------------------------------------------------------------------
	** Sign on.
	*/
	fprintf(stderr, "EALISTD; v1.00 (C)Copyright 1995, Novell Inc.  All rights reserved.\n");
	fprintf(stderr, "\n");

	/*------------------------------------------------------------------------
	**	Verify command line args.
	*/
  	if(argC != 2)
		{
		fprintf(stderr, "ERROR: Incorrect number of command line args.\n");
		goto END_ERR;
		}
	strupr(argV[1]);

	/*------------------------------------------------------------------------
	**	Initialize Novell Client library.
	*/
   rVal=NWCallsInit(
		/*	--	in		*/	NULL,
		/*	--	out	*/	NULL
		);
	switch(rVal)
		{
		case 0x0000:	/* Successful	*/
			break;

		case 0x0001:	
			fprintf(stderr, "ERROR:  NWCallsInit() reports UNABLE TO OBTAIN DOUBLE BYTE INFORMATION.\n");
			goto END_ERR;

		case 0x88FF:
			fprintf(stderr, "ERROR:  NWCallsInit() reports SHELL_FAILURE/REM_FAILURE.\n");
			goto END_ERR;

		default:
			fprintf(stderr, "ERROR:  NWCallsInit() reports undocumented error 0x%04X\n", rVal);
			goto END_ERR;
		}

	/*------------------------------------------------------------------------
	**	Extract the connection, dirHandle and newPath from the specified path.
	*/
   rVal=NWParseNetWarePath(
		/* I-	path			*/	argV[1],
		/*	-O	conn			*/	&fileCon,
		/*	-O	dirHandle	*/	&fileDirHandle,
		/*	-O	newPath		*/	filePath
		);
	switch(rVal)
		{
		case 0x0000:	/* Successful	*/
			break;

		case 0x880F:
			fprintf(stderr, "ERROR:  NWParseNetWarePath() reports NO CONNECTION TO SERVER.\n");
			goto END_ERR;

		case 0x883C:
			fprintf(stderr, "ERROR:  NWParseNetWarePath() reports NOT MY RESOURCE.\n");
			goto END_ERR;

		default:
			fprintf(stderr, "ERROR:  NWParseNetWarePath() reports undocumented error 0x%04X\n", rVal);
			goto END_ERR;
		}

	/*------------------------------------------------------------------------
	**	Prepare for EA search.
	*/
	rVal=NWNSGetMiscInfo(
		/*	I-	conn				*/ fileCon,
		/*	I-	dirHandle		*/	fileDirHandle,
		/*	I-	path				*/	filePath,
		/*	I-	destNmeSpace	*/	NAME_SPACE,
		/*	-O	idxStruct		*/	&fileIdx
		);
	switch(rVal)
		{
		case 0x0000:	/* Successful	*/
			break;

		case 0x8801:
			fprintf(stderr, "ERROR:  NWNSGetMiscInfo() reports INVALID CONNECTION.\n");
			goto END_ERR;

		case 0x8998:
			fprintf(stderr, "ERROR:  NWNSGetMiscInfo() reports VOLUME_DOES_NOT_EXIST.\n");
			goto END_ERR;

		case 0x899B:
			fprintf(stderr, "ERROR:  NWNSGetMiscInfo() reports BAD DIRECTORY HANDLE.\n");
			goto END_ERR;

		case 0x899C:
			fprintf(stderr, "ERROR:  NWNSGetMiscInfo() reports INVALID PATH.\n");
			goto END_ERR;

		default:
			fprintf(stderr, "ERROR:  NWNSGetMiscInfo() reports undocumented error 0x%04X\n", rVal);
			goto END_ERR;
		}


	/*------------------------------------------------------------------------
	** Print header.
	*/
	printf("Extended Attributes for %s:\n", argV[1]);

	/*------------------------------------------------------------------------
	** Scan through the EAs.
	*/
	eaCnt=0;
	rVal=NWFindFirstEA(
		/*	I-	conn			*/	fileCon,
		/*	I-	idxStruct	*/	&fileIdx,
		/*	-O	ffStruct		*/	&eaFfStruct,
		/*	-O	EAHandle		*/ &eaHandle,
		/*	-O EAName		*/	eaName
		);
	while(rVal == 0x0000)
		{
		++eaCnt;

   	rVal=NWOpenEA(
			/*	I-	conn			*/	fileCon,
			/*	I-	dirHandle	*/	fileDirHandle,
			/*	I-	path			*/	filePath,
			/*	I-	EAName		*/	eaName,
			/*	I-	nameSpace	*/	NAME_SPACE,
			/*	IO	EAHandle		*/	&eaHandle
			);
		
		switch(rVal)
			{
			case 0x0000:

   			rVal=NWReadEA(
					/*	IO	dirHandle	*/	&eaHandle,
					/*	I-	bufferSize	*/	EA_BUFFER_SIZE,
					/*	-O	buffer		*/	eaBuffer,
					/*	-O	totalEASize	*/	&eaSize,
					/*	-O	amountRead	*/	&eaScoop
					);
				switch(rVal)
					{
					case 0x0000:
					case 0x0001:
						printf("\n");
						printf("[%s]\n", eaName);
						printf("\n");
						Debug(eaSize, eaBuffer);
						break;

					default:
						fprintf(stderr, "ERROR: NWReadEA() returned 0x%04X\n", rVal);
						break;
					}

	 			rVal=NWCloseEA(&eaHandle);
				switch(rVal)
					{
					case 0x0000:
						break;

					case 0x89CF:
						fprintf(stderr, "ERROR: NWCloseEA() reports INVALID_EA_HANDLE.\n");
						break;

					case 0x89D3:
						fprintf(stderr, "ERROR: NWCloseEA() reports EA_VOLUME_NOT_MOUNTED.\n");
						break;

					default:
						fprintf(stderr, "ERROR: NWCloseEA() reports undocumented error: 0x%04X.\n", rVal);
						break;
					}

				
				break;

			case 0x8801:
				fprintf(stderr, "ERROR: NWOpenEA() reports INVALID CONNECTION.\n");
				break;

			case 0x8998:
				fprintf(stderr, "ERROR: NWOpenEA() reports VOLUME DOES NOT EXIST.\n");
				break;

			case 0x899B:
				fprintf(stderr, "ERROR: NWOpenEA() reports BAD DIRECTORY HANDLE.\n");
				break;

			case 0x899C:
				fprintf(stderr, "ERROR: NWOpenEA() reports INVALID PATH.\n");
				break;

			default:
				fprintf(stderr, "ERROR: NWOpenEA() reports undocumented error: 0x%04X.\n");
				break;
			}
		
		printf("\n");

		rVal=NWFindNextEA(
			/*	IO	ffStruct	*/	&eaFfStruct,
			/*	-O	EAHandle	*/	&eaHandle,
			/*	-O	EAName	*/	eaName
			);
		}

	switch(rVal)
		{
		case 0x0000:	/* This won't ever happen */
			break;

		case 0x0001:	/* End of EA list encountered. */
			break;
	  
		case 0x8801:
			fprintf(stderr, "ERROR: NWFind%sEA() reports INVALID CONNECTION.\n", eaCnt ? "Next" : "First");
			break;
  
		default:
			fprintf(stderr, "ERROR: NWFind%s() reports undocumented error: %d.\n", eaCnt ? "Next" : "First", rVal);
			break;
		}

	printf("%d Extended Attribute%s in file.\n", eaCnt, (eaCnt == 1) ? "" : "s");

END_ERR:

	exit(0);
	}
