/*****************************************************************************
**	File:	EALIST.C
**
**	Description:
**
** 	Sample NLM application that lists the Extended Attributes of a file.
**
**	Disclaimer:
**
**		Novell, Inc. makes no representations or warranties with respect to
**		any NetWare software, and specifically disclaims any express or
**		implied warranties of merchantability, title, or fitness for a
**		particular purpose.  
**
**		Distribution of any NetWare software is forbidden without the
**		express written consent of Novell, Inc.  Further, Novell reserves
**		the right to discontinue distribution of any NetWare software.
**
**		Novell is not responsible for lost profits or revenue, loss of use
**		of the software, loss of data, costs of re-creating lost data, the
**		cost of any substitute equipment or program, or claims by any party
**		other than you.  Novell strongly recommends a backup be made before
**		any software is installed.   Developer support for this software
**		may be provided at the discretion of Novell.
**
**	QMK386 options used:
**
**		None
**
**	Programmers:
**
**		Ini	Who					Firm
**		-----------------------------------------------------------------------
**		ABJ	Adam B. Jerome		Novell Developer Support.
**
**	History:
**
**		When		Who	What
**		-----------------------------------------------------------------------
**		01-12-95	ABJ	First code.
**
*/

/****************************************************************************
**	Compiler Setup.
*/
	/*------------------------------------------------------------------------
	**	ANSI
	*/
	#include <stdio.h> 	/* stderr, printf(), fprintf() */
	#include <stdlib.h>	/* exit() */
	#include <string.h>	/* strupr(), strlen() */
	#include <ctype.h>	/*	isprint()	*/
	#include <errno.h>	/* errno, ESUCCESS, EFAILURE */
								 	
	/*------------------------------------------------------------------------
	**	Novell
	*/
	#include <niterror.h>/* NetWareErrNo	*/
	#include <extattr.h>	/* OpenEA(), CloseEA(), ReadEA(), GetEAInfo(), EnumerateEA() */

/****************************************************************************
**	Print a buffer, Debug style.
*/
void Debug(long len, char *buffer)
	{
	char szTemp[80+1];
	long nCnt;
	int  lineCnt;

	printf("       |  0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F  0123456789ABCDEF\n");
	printf("-------+------------------------------------------------- ----------------\n");

	lineCnt=0;
	for(nCnt=0; nCnt<len; ++nCnt)
		{
		if(!(nCnt % 16))
			{
			if(nCnt)
				{
				++lineCnt;
				printf("  %s\n", szTemp);
				}

			printf("0x%03Xn |", lineCnt);
			memset(szTemp, '\0', sizeof(szTemp));
			}

		printf(" %02X", 0x00FF & buffer[nCnt]);
		szTemp[strlen(szTemp)] = (char)((isprint(buffer[nCnt])) ? buffer[nCnt] : '.');
		}

	while((nCnt % 16))
		{
		printf("   ");
		++nCnt;
		}
	printf("  %s\n", szTemp);

	return;
	}

/****************************************************************************
**	Program Start.
*/
void main(int argC, char *argV[])
	{
	int  				 		 rVal; 							/* Used to record function status return codes. */

   int 						 eaHandle	=	EFAILURE;	/* Handle for extended attributes context. */

	LONG						 eaTotalCnt;					/* Indicates number of EAs in the file.		*/
	LONG						 eaTotalDataSize;				/* Used to calculate the size of enumBuffer. */
	LONG						 eaTotalKeySize;				/*	Returned by GetEAInfo, but not used.		*/

	char						*enumBuffer	=	NULL;			/*	Pointer to the EA enumerated buffer. */
	LONG						 enumBufferSize;				/* Size of the enumBuffer. */
	LONG						 enumDataSize;					/* Number of bytes in the enumBuffer, as indicated by EnumerateEA() */
	LONG						 enumEAsInReply;				/* Number of EAs In enumBuffer. */

   T_enumerateEAnoKey 	*noKey;							/* Used to point at an T_enumerateEAnoKey structure in the eaBuffer. */
	LONG 						 offset;							/* Used to step through the eaBuffer. */

   char 						 eaKeyBuff[255];				/* Used to construct a zero-terminated string containing the keyValue. */
   char 						*eaBuffer	=	NULL;			/* Pointer to the EA value buffer. */
	LONG 						 eaBufferSize;					/* Size of the eaBuffer */
	int						 eaBufferBytes;				/* Number of bytes in the eaBuffer, as indicated by ReadEA() */
	LONG						 eaAccessFlags;				/*	Access Flags (set by WriteEA). */

	/*------------------------------------------------------------------------
	** Sign on.
	*/
	fprintf(stderr, "EALIST; v1.00 (C)Copyright 1995, Novell Inc.  All rights reserved.\n");
	fprintf(stderr, "\n");

	/*------------------------------------------------------------------------
	**	Verify command line args.
	*/
  	if(argC != 2)
		{
		fprintf(stderr, "ERROR: Incorrect number of command line args.\n");
		goto END_ERR;
		}
	strupr(argV[1]);

	/*------------------------------------------------------------------------
	** Print header.
	*/
	printf("Extended Attributes for %s:\n", argV[1]);

	/*------------------------------------------------------------------------
	** Open file for extended attribute (EA) I/O.
	*/
   eaHandle=OpenEA(
		/*	I-	path			*/	argV[1],
		/*	I-	reserved		*/	0
		);
	if(eaHandle == EFAILURE)
		{
		fprintf(stderr, "ERROR: OpenEA() reports errno[%d]/NetWareErrno[%d]\n", errno, NetWareErrno);
		goto END_ERR;
		}
	
	/*------------------------------------------------------------------------
	** Get info about EAs on this file.
	*/
	rVal=GetEAInfo(
		/*	I-	handle					*/	eaHandle,
		/*	-O	totalEAs					*/	&eaTotalCnt,
		/*	-O	eaTotalDataSizeOfEAs	*/	&eaTotalDataSize,
		/*	-O	totalKeySizeOfEas		*/	&eaTotalKeySize
		);
	switch(rVal)
		{
		case 0:
			break;

		default:
			fprintf(stderr, "ERROR: GetEAInfo() reports[%d]/errno[%d]/NetWareErrno[%d]\n", rVal, errno, NetWareErrno);
			goto END_ERR;
		}
	
	/*------------------------------------------------------------------------
	** Calculate size and allocate the enumBuffer.  EnumerateEA() requires
	**	that the dataBufferSize be a multiple of 128 bytes.
	*/
	if(eaTotalDataSize % 128)
		enumBufferSize=((eaTotalDataSize / 128) + 1) * 128;
	else
		enumBufferSize=eaTotalDataSize;

	enumBuffer=malloc(enumBufferSize);
	if(enumBuffer == NULL)
		{
		fprintf(stderr, "ERROR: Cannot allocate %lu bytes for enumDataBuf.\n", enumBufferSize);
		goto END_ERR;
		}

	/*------------------------------------------------------------------------
	** Get all EA key info.
	*/
	rVal=EnumerateEA(
		/*	I-	handle			*/	eaHandle,
		/*	I-	keyBuffer		*/	NULL,					/* Retrieve all EA information. */
		/*	-O	dataBuffer		*/	enumBuffer,	
		/*	I-	dataBufferSize	*/	enumBufferSize,
		/*	I-	startPosition	*/	0,
		/*	-O	dataSize			*/	&enumDataSize,
		/*	-O	enumEAsInReply		*/	&enumEAsInReply
		);
	switch(rVal)
		{
		case 0:
			break;

		case 215:
			fprintf(stderr, "ERROR: EnumerateEA() reports[%d]: ODD BUFFFER SIZE.\n", rVal, errno, NetWareErrno);
			goto END_ERR;

		default:
			fprintf(stderr, "ERROR: EnumerateEA() reports[%d]/errno[%d]/NetWareErrno[%d]\n", rVal, errno, NetWareErrno);
			goto END_ERR;
		}


	/*------------------------------------------------------------------------
	** Scan through the EAs.
	*/
   offset = 0;
   while(enumEAsInReply--)
      {
      noKey = (T_enumerateEAnoKey *)(enumBuffer + offset);
      
		memcpy(eaKeyBuff, &noKey->keyValue, noKey->keyLength);
		eaKeyBuff[noKey->keyLength] = '\0'; /* make key ASCIIZ */
      
      offset += noKey->keyLength + sizeof(T_enumerateEAnoKey) - 1;

		/*------------------------------------------------------------------------
		** Calculate size and allocate the eaBuffer.  ReadEA() requires
		**	that the bufferSize be a multiple of 128 bytes.
		*/
		if(noKey->valueLength % 128)
			eaBufferSize=((noKey->valueLength / 128) + 1) * 128;
		else
			eaBufferSize=noKey->valueLength;

		eaBuffer=malloc(enumBufferSize);
		if(enumBuffer == NULL)
			{
			fprintf(stderr, "ERROR: Cannot allocate %lu bytes for eaBuffer.\n", eaBufferSize);
			goto END_ERR;
			}

      /*---------------------------------------------------------------------
		**	Read the EA value.
		*/
   	eaBufferBytes=ReadEA(
			/*	I-	handle		*/	eaHandle,
			/*	I-	keyBuffer	*/	eaKeyBuff,
			/*	-O	dataBuffer	*/	eaBuffer,
			/*	I-	bufferSize	*/	eaBufferSize,
			/*	-O	accessFlags	*/	&eaAccessFlags
			);
		if(eaBufferBytes == EFAILURE)
			{
			fprintf(stderr, "ERROR: ReadEA() reports errno[%d]/NetWareErrno[%d]\n", errno, NetWareErrno);
			goto END_ERR;
			}

      /*---------------------------------------------------------------------
		**	Display the EA value.
		*/
		printf("\n");
		printf("[%s]\n", eaKeyBuff);
		printf("\n");
		Debug(eaBufferBytes, eaBuffer);
		printf("\n");
      
		/*------------------------------------------------------------------------
	 	**	Free the eaBuffer.
		*/
		if(eaBuffer != NULL)
			free(eaBuffer);
		}

	printf("%d Extended Attribute%s in file.\n", eaTotalCnt, (eaTotalCnt == 1) ? "" : "s");

END_ERR:

	if(enumBuffer != NULL)
		free(enumBuffer);

	if(eaHandle != EFAILURE)
		{
		rVal=CloseEA(eaHandle);
		if(rVal != ESUCCESS)
			fprintf(stderr, "ERROR: CloseEA() reports[%d]/errno[%d]/NetWareErrno[%d]\n", rVal, errno, NetWareErrno);
		}

	exit(0);
	}
